# -*- coding: utf-8 -*-

from Handlers import BaseHandler
from IPy import IP
import shutil
import os
#-------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------
class Interface(BaseHandler.BaseHandler):
	#=======================================================
	# 
	#=======================================================
	def __init__(self):
		self.config = '/etc/network/interfaces'
		self.pre_conf = '/tmp/interfaces'
		self.name = 'Interface'
		self.interface = ['eth0', 'eth1', 'eth2']
		self.addressFields = ['address', 'netmask']
		self.AUTO = 'auto %s \n'
		self.IFACE = 'iface %s inet %s\n'
		self.CMD = '\t%s %s\n'
		self.DNS = '\tdns-nameservers %s %s\n'
		self.BRIDGE = '\tbridge_ports eth0 eth1\n\tbridge_stp off\n\tbridge_maxwait 1\n'
		self.PRE = '\tpre-up   ifconfig $IFACE up\n\tpre-down ifconfig $IFACE down\n'
		self.GATEWAY = '\nup route add default gw %s\n'
	#=======================================================
	# 
	#=======================================================
	def Get(self, request):

		status = 200
		return self.readInterfaces(), status
	
	#=======================================================
	# 
	#=======================================================
	def Put(self, request):
		
		#print(request.json)
		status = self.validator(request.json)

		if status == 200:
			self.writeInterfaces(request.json)
			self.restartInterface(request.json)

		return status

	#=======================================================
	# 
	#=======================================================
	def readInterfaces(self):

		config = {}

		interfaces = open(self.config, "r")

		for interface in self.interface:
			config[interface] = {'getaddress':'', 'address':'', 'netmask':''}
		config['common'] = {'gateway':'', 'pridns':'','secdns':'', 'pppoe':'false'}

		for line in interfaces:
			if line.startswith('#'):
				pass
			else:

				if line.startswith('iface'):
					sline = line.split()
					interface = sline[1]
					if interface in config:
						config[interface]["getaddress"] = sline[3]
					else:
						interface = ''
				elif line.startswith('up route'):
					sline = line.strip('\n').split()
					config['common']['gateway'] = sline[5]
					
				elif line.isspace():
					pass
				elif interface:
					sline = line.split()
					if sline[0] == 'address':
						config[interface]["address"] = sline[1]
					elif sline[0] == 'netmask':
						config[interface]["netmask"] = sline[1]
					elif sline[0] == 'gateway':
						config[interface]["gateway"] = sline[1]
					elif sline[0] == 'dns-nameservers':
						config['common']["pridns"] = sline[1]
						if len(sline) > 2:
							config['common']["secdns"] = sline[2]

		if os.path.exists('/etc/network/interfaces.d/dsl-provider'):
			config['common']['pppoe'] = 'true'

		return config

	#=======================================================
	# 
	#=======================================================
	def writeInterfaces(self, args):

		config = open(self.config, "w")
		config.write("# interfaces(5) file used by ifup(8) and ifdown(8)\n# Include files from\ /etc/network/interfaces.d:\nsource-directory /etc/network/interfaces.d\n")
		config.write(self.AUTO % 'lo')
		config.write(self.IFACE % ('lo', 'loopback'))

		for interface in self.interface:
			config.write(self.AUTO % interface)
			config.write(self.IFACE % (interface, args[interface]['getaddress']))
			for key in self.addressFields:
				if args[interface][key]:
					config.write(self.CMD % (key, args[interface][key]))
			if args['common']['pridns']:
				config.write(self.DNS % ( args['common']['pridns'], args['common']['secdns']))
			#config.write(self.PRE)

		if args['common']['gateway']:
			config.write(self.GATEWAY % args['common']['gateway'])

		config.close()

	#=======================================================
	# 
	#=======================================================
	def restartInterface(self, args):

		for interface in self.interface:
			self.call(['ip', 'addr', 'flush', 'dev', interface])
		self.call("ifdown --exclude=lo -a && ifup --exclude=lo -a",  shell=True)
		self.service('dnsmasq', 'restart')
		if args['common']['gateway']:
			self.call('route add default gw %s' % args['common']['gateway'], shell=True)

	#=======================================================
	# 
	#=======================================================
	def validator(self, args):

		result = 400

		for interface in self.interface :
			if args[interface]['getaddress'] == 'static':
				if args[interface]['address'] != '0.0.0.0':
					if not self.check_ipaddr(args[interface]['address']):
						return result
				if args[interface]['netmask'] == '':
						return result

		# check reachable gateway 
		if args['common']['gateway']:
			for interface in self.interface:
				if args[interface]['getaddress'] == 'static':
					if (IP(args[interface]['address']).make_net(args[interface]['netmask'])).overlaps(args['common']['gateway']):
						result = 200
		else:
			result = 200

		return result
